@@ -177,8 +177,12 @@ class GPXRoute:
 
 		return mod_geo.Location( latitude = sum_lat / n, longitude = sum_lon / n )
 
-	def get_points( self ):
-		return self.points
+	def walk( self, only_points = False ):
+		for point_no, point in enumerate( self.points ):
+			if only_points:
+				yield point
+			else:
+				yield point, point_no
 
 	def move( self, latitude_diff, longitude_diff ):
 		for route_point in self.points:
@@ -280,12 +284,6 @@ class GPXTrackPoint( mod_geo.Location ):
 	# Position dilution of precision
 	position_dilution = None
 
-	# reloaded with reload_points_metadata():
-	__distance_from_track_start = None
-	__track_no = None
-	__segment_no = None
-	__point_no = None
-
 	def __init__( self, latitude, longitude, elevation = None, time = None, symbol = None, comment = None,
 			horizontal_dilution = None, vertical_dilution = None, position_dilution = None ):
 		mod_geo.Location.__init__( self, latitude, longitude, elevation )
@@ -422,11 +420,19 @@ class GPXTrack:
 
 		return Bounds( min_lat, max_lat, min_lon, max_lon )
 
-	def get_points( self ):
-		result = []
+	def walk( self, only_points = False ):
+		for segment_no, segment in enumerate( self.segments ):
+			for point_no, point in enumerate( segment.points ):
+				if only_points:
+					yield point
+				else:
+					yield point, segment_no, point_no
+
+	def get_points_no( self ):
+		result = 0
 
 		for track_segment in self.segments:
-			result += track_segment.get_points()
+			result += track_segment.get_points_no()
 
 		return result
 
@@ -592,14 +598,6 @@ class GPXTrack:
 
 		return mod_geo.Location( latitude = sum_lat / n, longitude = sum_lon / n )
 
-	def get_points_no( self ):
-		result = 0
-
-		for track_segment in self.segments:
-			result += track_segment.get_points_no()
-
-		return result
-
 	def smooth( self, vertical = True, horizontal = False, remove_extreemes = False ):
 		""" See: GPXTrackSegment.smooth() """
 		for track_segment in self.segments:
@@ -664,8 +662,19 @@ class GPXTrackSegment:
 		for track_point in self.points:
 			track_point.move( latitude_diff, longitude_diff )
 
-	def get_points( self ):
-		return self.points
+	def walk( self, only_points = False ):
+		""" Use this to iterate through points """
+		for point_no, point in enumerate( self.points ):
+			if only_points:
+				yield point
+			else:
+				yield point, point_no
+
+	def get_points_no( self ):
+		""" Number of points """
+		if not self.points:
+			return 0
+		return len( self.points )
 
 	def split( self, point_no ):
 		""" Splits this segment in two parts. Point #point_no remains in the first part. 
@@ -905,12 +914,6 @@ class GPXTrackSegment:
 			content += track_point.to_xml( version )
 		return mod_utils.to_xml( 'trkseg', content = content )
 
-	def get_points_no( self ):
-		""" Number of points """
-		if not self.points:
-			return 0
-		return len( self.points )
-
 	def get_nearest_location( self, location ):
 		""" Return the ( location, track_point_no ) on this track segment """
 		if not self.points:
@@ -1094,8 +1097,6 @@ class GPX:
 	min_longitude = None
 	max_longitude = None
 
-	__last_hash_value = None
-
 	def __init__( self, waypoints = None, routes = None, tracks = None ):
 		self.time = None
 
@@ -1122,39 +1123,6 @@ class GPX:
 		self.min_longitude = None
 		self.max_longitude = None
 
-	def reload_points_metadata( self ):
-		"""
-		Will reload GPXTrackPoint.__distance_from_track_start, __track_no, __segment_no,
-		__point_no.
-
-		Note that the reloa will be executed only if the hash (see __hash__()) of the track
-		is changed). Otherwise, once reloaded values are cached.
-		"""
-
-		if self.__last_hash_value == hash( self ):
-			return
-
-		mod_logging.debug( 'Reload' )
-
-		previous_point = None
-		distance = 0
-		for track_no in range( len( self.tracks ) ):
-			track = self.tracks[ track_no ]
-			for segment_no in range( len( track.segments ) ):
-				segment = track.segments[ segment_no ]
-				for point_no in range( len( segment.points ) ):
-					point = segment.points[ point_no ]
-					if previous_point:
-						distance += point.distance_3d( previous_point )
-					previous_point = point
-
-					point.__distance_from_track_start = distance
-					point.__track_no = track_no
-					point.__segment_no = segment_no
-					point.__point_no = point_no
-
-		self.__last_hash_value = hash( self )
-
 	def get_time_bounds(self):
 		"""
 		Returns the first and last found time in the track. 
@@ -1273,7 +1241,7 @@ class GPX:
 		min_distance = The minimum distance between two points
 		"""
 
-		points_no = self.get_points()
+		points_no = list( self.walk() )
 		if not max_points_no or points_no <= max_points_no:
 			return
 
@@ -1303,7 +1271,8 @@ class GPX:
 
 				track_segment.points = reduced_points
 
-		mod_logging.debug( 'Track reduced to %s points' % len( self.get_points() ) )
+		# TODO
+		mod_logging.debug( 'Track reduced to %s points' % self.get_track_points_no() )
 
 	def split( self, track_no, track_segment_no, track_point_no ):
 		track = self.tracks[ track_no ]
@@ -1326,11 +1295,23 @@ class GPX:
 				result += length
 		return result
 
-	def get_points( self ):
-		result = []
+	def walk( self, only_points = False ):
+		""" Use this to iterate through points """
+		for track_no, track in enumerate( self.tracks ):
+			for segment_no, segment in enumerate( track.segments ):
+				for point_no, point in enumerate( segment.points ):
+					if only_points:
+						yield point
+					else:
+						yield point, track_no, segment_no, point_no
+
+	def get_track_points_no( self ):
+		""" Number of track points, *without* route and waypoints """
+		result = 0
 
 		for track in self.tracks:
-			result += track.get_points()
+			for segment in track.segments:
+				result += len( segment.points )
 
 		return result
 
